"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var amqp = require("amqplib");
var FunctionRegistory_1 = require("./FunctionRegistory");
var timeouts;
var RMQBroker = /** @class */ (function () {
    function RMQBroker() {
    }
    RMQBroker.prototype.init = function (rmqConfig) {
        return new Promise(function (res, rej) {
            RMQBroker.rmqOptions = rmqConfig;
            amqp
                .connect(rmqConfig.URL)
                .then(function (connectedCon) {
                RMQBroker.CONN = connectedCon;
                RMQBroker.CONN.createChannel().then(function (ch) {
                    RMQBroker.CHAN = ch;
                    res();
                });
            })
                .catch(function (err) {
                console.log(err);
                rej(err);
            });
        });
    };
    RMQBroker.prototype.initFunctions = function (rmqConfig) {
        return new Promise(function (res, rej) {
            var FuncReg = new FunctionRegistory_1.default();
            FuncReg.initResponder(RMQBroker.CHAN, rmqConfig)
                .then(function () {
                FuncReg.initConsumer(RMQBroker.CHAN, rmqConfig).then(function () {
                    FuncReg.initGlobalConsumer(RMQBroker.CHAN, rmqConfig).then(function () {
                        res();
                    });
                });
            })
                .catch(function (err) {
                console.log(err);
                rej(err);
            });
        });
    };
    RMQBroker.prototype.publish = function (topic, message, persistMessage, expireMessageIn) {
        var buff = Buffer.from(JSON.stringify(message));
        var tempsplit = topic.split('.');
        var routingKey = tempsplit[1];
        var exchange = tempsplit[0];
        var ok = RMQBroker.CHAN.publish(exchange, routingKey, buff, {
            persistent: persistMessage,
            expiration: expireMessageIn,
        });
        console.log('[x] Event Published : ' + topic);
        return ok;
    };
    RMQBroker.prototype.subscribe = function (topic, callback) {
        throw new Error('Method not implemented.');
    };
    RMQBroker.prototype.rpc = function (topic, message, persistMessage, expireMessageIn) {
        return new Promise(function (res, rej) {
            console.log('[x] RPC Invoked : ' + topic);
            var buf = Buffer.from(JSON.stringify(message));
            RMQBroker.CHAN.assertQueue('', { exclusive: true, durable: false, autoDelete: true })
                .then(function (q) {
                var corr = generateUuid();
                RMQBroker.CHAN.consume(q.queue, function (msg) {
                    if (msg && msg.properties.correlationId === corr) {
                        var consumerTag = msg.fields.consumerTag;
                        console.log('[x] Response Received for : ' + q.queue);
                        clearTimeout(timeouts);
                        res(msg.content);
                        RMQBroker.CHAN.cancel(consumerTag);
                    }
                }, { noAck: true });
                RMQBroker.CHAN.sendToQueue(topic, buf, {
                    correlationId: corr,
                    replyTo: q.queue,
                    persistent: persistMessage,
                    expiration: expireMessageIn,
                });
                // return if no respose received from topic in 15 sec's
                timeouts = setTimeout(function () {
                    RMQBroker.CHAN.deleteQueue(q.queue);
                    res(false);
                }, RMQBroker.rmqOptions.RPC_TIMEOUT);
            })
                .catch(function (err) {
                console.log(err);
                rej(err);
            });
        });
    };
    RMQBroker.prototype.unsubscribe = function (topic) {
        throw new Error('Method not implemented.');
    };
    RMQBroker.prototype.onConnectionClose = function (handleCloseEvent) {
        RMQBroker.CONN.on('close', function () {
            handleCloseEvent();
        });
    };
    RMQBroker.prototype.onConnectionError = function (handleErrorEvent) {
        RMQBroker.CONN.on('error', function (err) {
            handleErrorEvent(err);
        });
    };
    return RMQBroker;
}());
exports.default = RMQBroker;
// temp for public code.
function generateUuid() {
    return Math.random().toString() + Math.random().toString() + Math.random().toString();
}
